#include <stdio.h>	/* drawsoln.c: group solutions by duality, and output	*/
#include <stdlib.h>	/* in PostScript; expects up to 26 solutions, but is	*/
#include <assert.h>	/* easily extended. Singletons are handled correctly. */
							/* Pseudosolutions are drawn in dark gray.				*/
#define N	6
#define NF	720
#define M0  3
#define SHIFTY	60
#define SHIFTX	66
#define SINGLESHIFTX 33
#define TEXTWIDTH	5

typedef struct Row {
	int r[M0];
} Row;
   
typedef int Cube[M0*M0*M0];

Row order[N] = {
{{0, 1, 2}},
{{0, 2, 1}},
{{1, 0, 2}},
{{1, 2, 0}},
{{2, 0, 1}},
{{2, 1, 0}}
};

Row pows[N] = {
{{1, 3, 9}},
{{1, 9, 3}},
{{3, 1, 9}},
{{3, 9, 1}},
{{9, 1, 3}},
{{9, 3, 1}}
};

int flipt[2][N] = {
{0, 1, 2, 3, 4, 5},
{2, 3, 0, 1, 5, 4}
/*{5, 4, 3, 2, 1, 0}*/};

#define fl(v, b)	((b) ? (M0 - 1 - (v)) : v)

/*Same routine as in symm.c*/
void get_cube(Cube c, Cube out, int p, int b1, int b2, int b3) 
{
	int i, j, k, l, v;
	Row r;
	for (i = 0; i < M0; ++i)
		for (j = 0; j < M0; ++j)
			for (k = 0; k < M0; ++k) {
				v = c[fl(i, b1)*pows[p].r[0] + fl(j, b2)*pows[p].r[1] + fl(k, b3)*pows[p].r[2]];
				for (l = 0; l < M0; ++l)
					r.r[l] = order[v].r[order[p].r[l]];
				for (l = 0; l < N; ++l)
					if (r.r[0] == order[l].r[0] && r.r[1] == order[l].r[1] && r.r[2] == order[l].r[2])
						break;
				assert(l < N);
				out[i + 3*j + 9*k] = l;
			}
}

int dim[M0] = {12, 18, 24};

int length[N];
int width[N];
int height[N];

typedef struct Point {
	int x, y;
} Point;

Point cells[14] = {
{0, 0},
{135, 0},
{270, 0},
{405, 0},
{0, 195},
{135, 195},
{270, 195},
{405, 195},
{0, 390},
{135, 390},
{270, 390},
{405, 390},
{135, 585},
{270, 585}
};

typedef struct Point3D {
	int x, y, z;
} Point3D;

typedef struct Block {
	Point3D min, max;
} Block;

#define ce c[i*M0*M0 + j*M0 + k]

int in(Block *b, int j, Point3D u, Point3D v) {
	int i;
	for (i = 0; i < M0*M0*M0; ++i)
		if (i != j && ((b[i].min.x < u.x && u.x < b[i].max.x) || (b[i].min.x < v.x && v.x < b[i].max.x) || (b[i].min.x == u.x && b[i].max.x == v.x)) &&
		((b[i].min.y < u.y && u.y < b[i].max.y) || (b[i].min.y < v.y && v.y < b[i].max.y)|| (b[i].min.y == u.y && b[i].max.y == v.y)) &&
		((b[i].min.z < u.z && u.z < b[i].max.z) || (b[i].min.z < v.z && v.z < b[i].max.z)|| (b[i].min.z == u.z && b[i].max.z == v.z)))
			return 1;
	return 0;
}

int is_bad_cube(Cube c)
{
	Block b[M0*M0*M0];
	int hMap[M0][M0], wMap[M0], lMap;
	int i, j, k, cb = 0;
	
	for (i = 0; i < M0; ++i)
		for (j = 0; j < M0; ++j)
			hMap[i][j] = 0;

	for (i = 0; i < M0; ++i) { 
		for (j = 0; j < M0; ++j)
			wMap[j] = 0;
		for (j = 0; j < M0; ++j) {
			lMap = 0;
			for (k = 0; k < M0; ++k) {
				b[cb].min.x = lMap;
				b[cb].min.y = wMap[k];
				b[cb].min.z = hMap[j][k];
				b[cb].max.x = (lMap += length[ce]);
				b[cb].max.y = (wMap[k] += width[ce]);
				b[cb].max.z = (hMap[j][k] += height[ce]);
				cb++;
			}
		}
	}	

	for (i = 0; i < M0*M0*M0; ++i)
		if (in(b, i, b[i].min, b[i].max))
			return 1;
	
	return 0;
}

void rect(FILE *out, int c, int x, int y)
{
	printf("%d %d %d %d rectstroke\n", x, y, length[c], width[c]);
}

void fill_rect(FILE *out, int c, int x, int y)
{
	printf("0.4 0.4 0.4 setrgbcolor %d %d %d %d rectfill 0.0 0.0 0.0 setrgbcolor\n", 
		x, y, length[c], width[c]);
}

void print_cube(FILE *out, Cube c, int xo, int yo)
{
	int bad_cube;
	int curY[M0], curX, cur;
	int i, j, k;
	
	bad_cube = is_bad_cube(c);
	
	for (i = 0; i < 3; ++i) {
		for (j = 0; j < 3; ++j)
			curY[j] = 0;
		for (j = 0; j < 3; ++j) {
			curX = 0;		
			for (k = 0; k < 3; ++k) {
				cur = ce;
				if (bad_cube)
					fill_rect(out, cur, xo + curX, yo + curY[k]);
				rect(out, cur, xo + curX, yo + curY[k]);
				curX += length[cur];
				curY[k] += width[cur];
			}
		} 
		yo += SHIFTY;
	}
}

void putsid(int x, int y, int sid)
{
	printf("/Times-Roman findfont 10.00 scalefont setfont %d %d moveto (%d) show\n", x + SHIFTX - 7, y - 4, sid);
}

void header()
{
	printf("%%!\n40 15 translate\n");
}

int tw(int sid) 
{
	if (sid > 10)
		return TEXTWIDTH*2;
	else 
		return TEXTWIDTH;
}

int main()
{
	Cube *f, out, in;
	int i, j, k, l, s, t, count;
	int cc = 0, sid = 0;

	for (i = 0; i < N; ++i) {
		length[i] = dim[order[i].r[0]];
		width[i] = dim[order[i].r[1]];
		height[i] = dim[order[i].r[2]];
	}

	scanf("%d", &count);
	f = malloc(count * sizeof(Cube));
	for (i = 0; i < count; ++i)
		for (j = 0; j < M0*M0*M0; ++j)
			scanf(" %d", &(f[i][j]));
	
	header(stdout);
	/*Same basic loop as in symm.c, except iteration over t and flipping added*/
	for (s = 0; s < count; ++s) {
		if (f[s][0] < 0) 
			continue;
		for (t = 0; t < 2; ++t) {
			for (i = 0; i < M0*M0*M0; ++i)
				in[i] = flipt[t][f[s][i]];
			for (i = 0; i < N; ++i)
				for (j = 0; j < 8; ++j)
					if (t != 0 || i != 0 || j != 0) {
						get_cube(in, out, i, j & 0x1, (j >> 1) & 0x1, j >> 2);
						for (k = 0; k < count; ++k) {
							for (l = 0; l < M0*M0*M0; ++l)
								if (f[k][l] < 0 || f[k][l] != out[l])
									break;
							if (l == M0*M0*M0) {
								printf("%%%d flip-symmetric to %d; %d %d %d\n", s, k, t, i, j);
								
								if (k != s) {
									print_cube(stdout, f[s], cells[cc].x + 5, cells[cc].y + 5); sid++;
									putsid(cells[cc].x - tw(sid), cells[cc].y, sid);
									print_cube(stdout, f[k], cells[cc].x + SHIFTX + 5, cells[cc].y + 5); sid++;
									putsid(cells[cc].x + 12, cells[cc].y, sid);
								} else {
									print_cube(stdout, f[s], cells[cc].x + SINGLESHIFTX + 5, cells[cc].y + 5); sid++;
									putsid(cells[cc].x + 6 - tw(sid)/2, cells[cc].y, sid);
								}
								
								assert(cc < 14);
								cc++;
								f[k][0] = -1 - f[k][0]; /*Mark both single and dual with negative*/
								break;
							} 
						}
					}
		}
	}
	printf("showpage\n");
	return 0;
}
