#include <stdio.h>	/* symm.c: Discard symmetric variants from a list of cubes. */
#include <stdlib.h>	/* Also discard cubes, which don't have the full list of    */
#include <assert.h>	/* symmetries present in the file. This is explained below. */

#define N	6
#define M0  3

typedef struct Row {
	int r[M0];
} Row;
typedef int Cube[M0*M0*M0];

Row order[N] = {
{{0, 1, 2}},
{{0, 2, 1}},
{{1, 0, 2}},
{{1, 2, 0}},
{{2, 0, 1}},
{{2, 1, 0}}
};
Row pows[N] = { /*Just 3^order[i].r[j] precomputed for convenience*/
{{1, 3, 9}},
{{1, 9, 3}},
{{3, 1, 9}},
{{3, 9, 1}},
{{9, 1, 3}},
{{9, 3, 1}}
};
int flipt[2][N] = { /*I didn't expect it to be just 5-i :)*/
{0, 1, 2, 3, 4, 5},
{5, 4, 3, 2, 1, 0}};

#define fl(v, b)	((b) ? (M0 - 1 - (v)) : v)

/*
Transform the cube in c, storing the result in out p is an index into the
array order, specifying how to reorder the axes (rotation) b1-b3 are 0 or 1,
specifying the direction, in which the axes are traversed.
*/
void get_cube(Cube c, Cube out, int p, int b1, int b2, int b3) 
{
	int i, j, k, l, v;
	Row r;
	for (i = 0; i < M0; ++i)
		for (j = 0; j < M0; ++j)
			for (k = 0; k < M0; ++k) {
				v = c[fl(i, b1)*pows[p].r[0] + fl(j, b2)*pows[p].r[1] + fl(k, b3)*pows[p].r[2]];
				for (l = 0; l < M0; ++l)
					r.r[l] = order[v].r[order[p].r[l]];
				for (l = 0; l < N; ++l)
					if (r.r[0] == order[l].r[0] && r.r[1] == order[l].r[1] && r.r[2] == order[l].r[2])
						break;
				assert(l < N); 
				out[i + 3*j + 9*k] = l;
			}
}

/*Print out a cube on a single line*/
void print_cube(FILE *out, Cube c)   
{
	int i, j, k;
	for (i = 0; i < 3; ++i) {
		for (j = 0; j < 3; ++j) {
			for (k = 0; k < 3; ++k) {
				fprintf(out, "%d", c[i*M0*M0 + j*M0 + k]);
				if (k != 2) putc(' ', out);
			}
			if (j != 2) putc('\t', out);
		} 
		fprintf(out, "  "); /*use "; " for readability*/
	}
	putc('\n', out);
}

int main()
{
	Cube *f, out;
	int i, j, k, l, s, count;

	/*Load cubes from file*/
	scanf("%d", &count);
	f = malloc(count * sizeof(Cube));
	for (i = 0; i < count; ++i)
		for (j = 0; j < M0*M0*M0; ++j)
			scanf(" %d", &(f[i][j]));
	
	/*Go through all cubes*/
	for (s = 0; s < count; ++s) {
		if (f[s][0] >= 0) /*If this wasn't thrown out as symmetrical to something else*/
			/*Iterate over all 48 symmetries of the cube*/
			for (i = 0; i < N; ++i)
				for (j = 0; j < 8; ++j)
					/*We know that f[s], with no rotation and no reflection occurs in the list :)*/
					if (i != 0 || j != 0) { 
						get_cube(f[s], out, i, j & 0x1, (j >> 1) & 0x1, j >> 2);
						/*Search the as of yet uncategorized cubes*/
						for (k = s; k < count; ++k) { 
							/*Compare element-by-element; bail out if different*/
							for (l = 0; l < M0*M0*M0; ++l)
								if (f[k][l] != out[l])
									break;
							/*If the same, mark the duplicate with a negative first element*/
							if (l == M0*M0*M0) {
								f[k][0] = -1;
								break;
							} 
						}
						/*If we didn't find anything, it must be one of those cubes that 
						only looks valid in some orientations (this occurs because we don't
						check for corner overlaps -- some overlaps end up looking like corner
						ones from a few orientations). We don't want such cubes, nor 
						do we care for any of their symmetries. */
						if (k == count) {
							f[s][0] = -1;
							goto continue_s_loop;
						}
					}
continue_s_loop:
		;
	}
	
	printf("26\n"); /*26 solnutions for "drawsoln.c", I cheat for convenience.*/ 
	/*Print out the non-duplicated cubes.*/
   for (s = 0; s < count; ++s)
		if (f[s][0] >= 0)
      	print_cube(stdout, f[s]);

	return 0;
}
